/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */

package com.ibm.trackntrace.barcode.parser;

public class Constants {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";
	
	public static final int APPLICATION_IDENTIFIER_LEN=2; //00 represents SSCC, 01 GTIN
	public static final int APP_ID_INDEX_START = 0;
	public static final int APP_ID_PREFIX_INDEX_END = 2;
	

	public class SSCC18Constants{
		// 18 digits long, 1: package indicator, 2-17: manf code, 18: check digit
		
		public static final int CASE_FILTER_VALUE = 0;
		public static final int PALLET_FILTER_VALUE = 2;
		

		public static final int CHECK_DIGIT_LEN=1;
		public static final int SSCC18_LEN=20; // SSCC=18, Application Identifier=2
		
		public static final String AI_SSCC="00";
		
		public static final int PACKAGING_TYPE_INDEX_START = 2;
		public static final int PACKAGING_TYPE_INDEX_END = 3;
		
		public static final int COMPANY_PREFIX_INDEX_START = 3;
		public static final int COMPANY_PREFIX_INDEX_END = 10;
		
		public static final int SERIAL_NUMBER_INDEX_START = 10;
		public static final int SERIAL_NUMBER_INDEX_END = 19;
		
		static public final String EPC_URN_PREFIX = "urn:epc:id:sscc:";
		
	}
	public class PharmaSGTIN14Constants{
		
		/* Case Tag barcode can be upto 45 digits in length.
		AI - GTIN  - 2 (01-GTIN)
		Packaging Indicator - 1
		NDC marker - 2 (03)
		NDC - 11 (10 digit NDC + check digit)
		AI - Expiry - 2 (17)
		Expiry Date - 6
		AI Lot# - 2 (10)
		Lot# - 1-20
		AI Serial # -2 (21)
		Serial # - 1-20
		*/
		
		
		public static final int APPLICATION_IDENTIFIER_LEN=2; //00 represents SSCC, 01 GTIN
		public static final String VARIABLE_ELEMENT_FIELD_SEPERATOR="~";


		public static final int CHECK_DIGIT_LEN=1;
		
		
		public static final String AI_SGTIN="01";
		public static final String AI_SERIAL_NUMBER="21";
		public static final String AI_EXPIRY="17";
		public static final String AI_LOT_NO="10";
		
		public static final int PHARMA_SGTIN14_MIN_LEN=19; /// AI-2,PackgeIndicator-1,NDCMarker-2,NDC with check digit-11, AI(serialNumber)-2, Serial #-1 = 19
		
		public static final int AFTER_EXPIRY_DATE_AI_MIN_LEN=9; // expiry date=6, AI(21) + atleast 1 digit serial Number

		
		public static final int AI_SGTIN_INDEX_START = 0;
		public static final int AI_SGTIN_INDEX_END = 2;
		
		public static final int PACKAGING_TYPE_INDEX_START = 2;
		public static final int PACKAGING_TYPE_INDEX_END = 3;
		
		public static final int NDC_INDEX_START = 5;
		public static final int NDC_INDEX_END = 16;
		
		public static final int SERIAL_NUMBER_MIN_LENGTH = 1;
		public static final int SERIAL_NUMBER_MAX_LENGTH = 20;
		
		public static final int EXPIRY_DATE_LENGTH = 6;
		
		public static final int LOT_NUMBER_MIN_LENGTH  = 1;
		public static final int LOT_NUMBER_MAX_LENGTH  = 20;
		
		public static final int AI_LENGTH = 2;
		
		//TODO - Update these with actual values
		public static final int UNIT_FILTER_VALUE   = 1;
		public static final int CASE_FILTER_VALUE   = 6;
		public static final int PALLET_FILTER_VALUE = 3;
		public static final int SLEEVE_FILTER_VALUE = 4;
		
		public static final String VALID_NDC_LIST="ndc_list.xml";
		
		
	}

}
