/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.ndc_mapper.utils;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

/**
 * @author nick
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class XmlGenerator extends JDialog implements ActionListener {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String ID_OPEN_RAW_BUTTON = "OpenRaw";
	private String ID_WRITE_XML_BUTTON = "WriteXml";
	
	private javax.swing.JPanel jContentPane = null;

	private JLabel jTopSpacer = null;
	private JButton jOpenRawButton = null;
	private JLabel jLowerSpacer = null;
	private JButton jWriteXmlFileButton = null;

	private ArrayList<String> ndcList = null;
	
	/** create xml file
	 * 
	 */
	private void saveXmlFile()  {
		JFileChooser chooser = new JFileChooser();
        int returnVal = chooser.showSaveDialog(this);
        
        // Create header of form:
        // <?xml version="1.0" encoding="windows-1252"?>
        // <NDCList>
        String xmlHeader = "<?xml version=\"1.0\" encoding=\"windows-1252\"?>\r\n<"+
			Constants.ELEMENT_NDC_LIST+">\r\n";

        // Create tail of form:  </NDCList>
        String xmlTail = "</"+Constants.ELEMENT_NDC_LIST+">\r\n";
        
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = chooser.getSelectedFile();
            //this is where a real application would save the file.
            try {
                BufferedWriter out = new BufferedWriter(new FileWriter(file.getAbsolutePath()));
                
                out.write(xmlHeader);
                Iterator it = ndcList.listIterator();
                while (it.hasNext())  {
                	String ndcElement = (String)it.next();
                	out.write(ndcElement+"\r\n");
                }
                out.write(xmlTail);
                
                out.close();
            } catch (IOException e) {
    			JOptionPane.showMessageDialog( this, "I/O Exception. Exception message is:\n"+e.getMessage(),
    					    	"I/O Exception", JOptionPane.ERROR_MESSAGE,	null);
            }
        } 
	}
	private boolean loadRawFile()  {

		File rawFile = null;
		ndcList = new ArrayList<String>();  // create new list each time open file.  
		
		//----------------------------
		// Prompt for a filename:
		JFileChooser chooser = new JFileChooser();
		chooser.setMultiSelectionEnabled(false);

		// Showe dialog, return if user did not click OK/Load
		if (JFileChooser.APPROVE_OPTION != chooser.showOpenDialog(null))  {
	    	return false;
	    }
      	// Get the File from the chooser:
		rawFile = chooser.getSelectedFile();

		//----------------------------
		// open, read the file into an arrayList:
        BufferedReader in;
		try {
			in = new BufferedReader(new FileReader(rawFile.getAbsoluteFile()));
			String str;
			while ((str = in.readLine()) != null) {

				// From http://www.fda.gov/cder/ndc/index.htm:
				// LBLCODE  NOT NULL CHAR(6) COL: 9-14
				// PRODCODE NOT NULL CHAR(4) COL: 16-19
				String lblcode = str.substring(8,14);
				String prodcode = str.substring(15,19);
				
				// product may have leading '*' char, strip if present
				if ( prodcode.charAt(0) == '*')  {
					System.err.println("Stripping leading star from \""+prodcode+"\"");
					prodcode = prodcode.substring(1);
				}
				// Create string of form: <ndc lblcode="000002" prodcode="1407"/>
				String xmlstr = "<"+
						Constants.ELEMENT_NDC+ " "+    
						Constants.ATTR_LABELER+"=\""+lblcode+"\" " +
						Constants.ATTR_PRODUCT+"\""+prodcode+"\"/>";
				ndcList.add(xmlstr);
			}

		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog( this, "Odd, unable to find file you selected. exception msg is:\n"+
					e.getMessage(),	"File not found",JOptionPane.ERROR_MESSAGE,	null);
	    	return false;
		}
		catch (IOException e) {
			JOptionPane.showMessageDialog( this, "I/O error. exception msg is:\n"+
					e.getMessage(),	"I/O Exception",JOptionPane.ERROR_MESSAGE,	null);
	    	return false;
		}

		// We have a list, enable "WriteXML" button so user can write to disk
		getJWriteXmlFileButton().setEnabled(true);
    	return true;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		
		String userChoice = e.getActionCommand();
		
		// Take action depending on what chosen:
		if ( userChoice.equals(ID_OPEN_RAW_BUTTON) )  {
			loadRawFile();
			
		} else if (userChoice.equals(ID_WRITE_XML_BUTTON) )  {
			saveXmlFile();
		} else {
			JOptionPane.showMessageDialog( this, "Ignoring Unknown action command \""+userChoice+"\"",
					"Unknown Command ID",JOptionPane.ERROR_MESSAGE,	null);
		}
	}


	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJOpenRawButton() {
		if (jOpenRawButton == null) {
			jOpenRawButton = new JButton();
			jOpenRawButton.setText("Open Raw File...");
			jOpenRawButton.setActionCommand(ID_OPEN_RAW_BUTTON);
			jOpenRawButton.addActionListener(this);

		}
		return jOpenRawButton;
	}
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJWriteXmlFileButton() {
		if (jWriteXmlFileButton == null) {
			jWriteXmlFileButton = new JButton();
			jWriteXmlFileButton.setText("Write to XML File...");
			jWriteXmlFileButton.setActionCommand(ID_WRITE_XML_BUTTON);
			jWriteXmlFileButton.setEnabled(false);
			jWriteXmlFileButton.addActionListener(this);

		}
		return jWriteXmlFileButton;
	}
  	public static void main(String[] args) {
	}
	/**
	 * This is the default constructor
	 */
	public XmlGenerator() {
		super();
		initialize();

		// If don't set this then JFrame(and JVM) hangs around....
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		try {
		    UIManager.setLookAndFeel(
		        UIManager.getSystemLookAndFeelClassName());
		} catch (UnsupportedLookAndFeelException ex) {
		  System.out.println("Unable to load native look and feel");
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	

		this.setSize(300,200);
		this.setContentPane(getJContentPane());
	}
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJContentPane() {
		if(jContentPane == null) {
			jTopSpacer = new JLabel();
			jLowerSpacer = new JLabel();
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			jContentPane = new javax.swing.JPanel();
			jContentPane.setLayout(new GridBagLayout());
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.gridy = 0;
			jTopSpacer.setText(" ");
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 2;
			jLowerSpacer.setText(" ");
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.gridy = 3;
			jContentPane.add(jTopSpacer, gridBagConstraints1);
			jContentPane.add(getJOpenRawButton(), gridBagConstraints2);
			jContentPane.add(jLowerSpacer, gridBagConstraints3);
			jContentPane.add(getJWriteXmlFileButton(), gridBagConstraints4);
		}
		return jContentPane;
	}
}
