/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.actions;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.ibm.trackntrace.forms.FormData;
import com.ibm.trackntrace.utils.Constants;
import com.ibm.trackntrace.utils.RoleDefs;
import com.ibm.trackntrace.utils.RoleValidation;


/**
 * @version 	1.0
 * @author
 */
public class MenuSelectionAction extends BXActionBase {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";
	static {
		log = Logger.getLogger(Constants.LOG_MENUS);
	}
	
    public ActionForward execute(ActionMapping mapping, FormData formData,
            HttpServletRequest request, HttpServletResponse response, ActionErrors errors)
            throws Exception {

        ActionForward forward = new ActionForward(); // return value
        String target = null;
        String screen = formData.getScreen();
        String command = formData.getCommand();
        String user = formData.getUserName();
        
        log.entering(getClass().getName(), "execute", formData); //$NON-NLS-1$
        
        if (user == null || user == "") {
        	user = request.getRemoteUser();
        	formData.setUserName(user);
           	log.log(Level.INFO, "menuSelectionAction: user ='" + user + "'");
        	//if (formData.getUserName() == null) {
        	//	target="GotoMainMenu";
            //	log.finest("User name is empty - forcing main menu to trigger WAS security");
        	//}
        }

        String station = formData.getStation();
        
    	log.log(Level.INFO, "station='" + station + "'");
    	

    	if (target != null) {
    		// if station or printer are unspecified then don't let them off the mainMenu
    		
    	} else if (command == null || screen == null) {
        	formData.setLastError("Transient internal error - check server logs.");
        	target="GotoMainMenu";
        	
        } else if (command.equals("MainMenu")) {
        	target="GotoMainMenu";
        	
        } else if (screen.equals("MainMenu")) { //$NON-NLS-1$
        	if (command.equals("Shipping")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.SHIPPER) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="ShippingGetOrder"; //$NON-NLS-1$
        	} else if (command.equals("Receiving")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.RECEIVER) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="ReceivingScanPallet"; //$NON-NLS-1$
        	} else if (command.equals("SplitPallet")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.SPLITTER) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="SplitPalletScanNewPallet"; //$NON-NLS-1$
        	} else if (command.equals("Samples")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.SAMPLER) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="SamplesMenu"; //$NON-NLS-1$
        	} else if (command.equals("Returns")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.RETURNS) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="ReturnsMenu"; //$NON-NLS-1$
        	} else if (command.equals("Reprint")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.REPRINT) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="ReprintScanItem"; //$NON-NLS-1$
        	} else if (command.equals("Damaged")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.DAMAGED) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="DamagedScanItem"; //$NON-NLS-1$
        	} else if (command.equals("Recommission")) { //$NON-NLS-1$
            	if (RoleValidation.validate(request, RoleDefs.RECOMMISSION) == false) {
            		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED + ": " + command);
            		target="GotoMainMenu";
            	} else target="RecommissionMenu"; //$NON-NLS-1$
        	} else if (command.equals("Logout")) { //$NON-NLS-1$
        		formData.setUserName("");
        		formData.setUserId("");
        		target="GotoMainMenu"; //$NON-NLS-1$
        	}
        	
        } else if (screen.equals("Shipping")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.SHIPPER) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("GotPallet")) { //$NON-NLS-1$
        		target="ShippingScanPallet"; //$NON-NLS-1$
        	} else if (command.equals("GotOrder")) { //$NON-NLS-1$
        		target="ShippingScanPallet"; //$NON-NLS-1$
        	} 
        	
        } else if (screen.equals("ShippingShowPallet")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.SHIPPER) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} 
        	
        } else if (screen.equals("Receiving")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.RECEIVER) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("GotPallet")) { //$NON-NLS-1$
        		target="ReceivingShowPallet"; //$NON-NLS-1$
        	} 

        } else if (screen.equals("SplitPallet")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.SPLITTER) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("GetSSC")) { //$NON-NLS-1$
        		target="SplitPalletScanNewPallet"; //$NON-NLS-1$
        	} else if (command.equals("GotPallet")) { //$NON-NLS-1$
        		target="SplitPalletScanCase"; //$NON-NLS-1$
        	} 

        } else if (screen.equals("SamplesMenu")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.SAMPLER) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("SampleCases")) { //$NON-NLS-1$
        		target="SamplesScanCase"; //$NON-NLS-1$
        	} else if (command.equals("SampleUnits")) { //$NON-NLS-1$
        		target="SamplesScanUnit"; //$NON-NLS-1$
        	} 
        	
        } else if (screen.equals("ReturnsMenu")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.RETURNS) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("ReturnCases")) { //$NON-NLS-1$
        		target="ReturnsScanCase"; //$NON-NLS-1$
        	} else if (command.equals("ReturnUnits")) { //$NON-NLS-1$
        		target="ReturnsScanUnit"; //$NON-NLS-1$
        	} 
        	
        } else if (screen.equals("Samples")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.SAMPLER) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("GotCase")) { //$NON-NLS-1$
        		target="SamplesScanCase"; //$NON-NLS-1$
        	} else if (command.equals("GotUnit")) { //$NON-NLS-1$
        		target="SamplesScanUnit"; //$NON-NLS-1$
        	} 
        	
        } else if (screen.equals("Damaged")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.DAMAGED) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("GotItem")) { //$NON-NLS-1$
        		target="DamagedScanItem"; //$NON-NLS-1$
        	}
        	
        } else if (screen.equals("Reprint")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.REPRINT) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("GotItem")) { //$NON-NLS-1$
        		target="ReprintScanItem"; //$NON-NLS-1$
        	} 
          	
        } else if (screen.equals("Recommission")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.RECOMMISSION) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoMainMenu";
        	} else if (command.equals("GotPallet")) { //$NON-NLS-1$
        		target="RecommissionScanCase"; //$NON-NLS-1$
        	} else if (command.equals("GotCase")) { //$NON-NLS-1$
        		target="RecommissionScanCase"; //$NON-NLS-1$
        	} 
        	
        } else if (screen.equals("DesktopMenu")) { //$NON-NLS-1$
        	if (RoleValidation.validate(request, RoleDefs.ADMIN) == false) {
        		formData.setLastError(Constants.ROLE_NOT_AUTHORIZED);
        		target="GotoDesktopMenu";
        		
        		// desktop main menu items
        	} else if (command.equals("PrintAdmin")) { //$NON-NLS-1$
        		target="DesktopPrintAdmin"; //$NON-NLS-1$

        	} else if (command.equals("ProductCodes")) { //$NON-NLS-1$
        		target="DesktopProductAdmin"; //$NON-NLS-1$
		
        	} else if (command.equals("ProductLabels")) { //$NON-NLS-1$
        		target="DesktopLabelAdmin"; //$NON-NLS-1$


        		// PrintAdmin sub screens
        	} else if (command.equals("DesktopAddPrintTemplate")) { //$NON-NLS-1$
        		target="DesktopAddPrintTemplate"; //$NON-NLS-1$
        		
        	} else if (command.equals("DesktopEditPrintTemplate")) { //$NON-NLS-1$
        		target="DesktopEditPrintTemplate"; //$NON-NLS-1$
        		
        	} else if (command.equals("DesktopDeletePrintTemplate")) { //$NON-NLS-1$
        		target="DesktopDeletePrintTemplate"; //$NON-NLS-1$
        		
        	} else if (command.equals("DesktopListAllPrintTemplate")) { //$NON-NLS-1$
        		target="DesktopListAllPrintTemplate"; //$NON-NLS-1$


        		// ProductCodes sub screens
        	} else if (command.equals("DesktopAddProduct")) {
        		target="DesktopAddProduct";
        	
        	} else if (command.equals("DesktopEditProduct")) { //$NON-NLS-1$
        		target="DesktopEditProduct"; //$NON-NLS-1$
        		
        	} else if (command.equals("DesktopDeleteProduct")) { //$NON-NLS-1$
        		target="DesktopDeleteProduct"; //$NON-NLS-1$
        		
        	} else if (command.equals("DesktopListAllProduct")) { //$NON-NLS-1$
        		target="DesktopListAllProduct"; //$NON-NLS-1$


        		// Product Labels sub screens
        	} else if (command.equals("DesktopAddLabel")) {
        		target="DesktopAddLabel";
        	
        	} else if (command.equals("DesktopEditLabel")) { //$NON-NLS-1$
        		target="DesktopEditLabel"; //$NON-NLS-1$
        		
        	} else if (command.equals("DesktopDeleteLabel")) { //$NON-NLS-1$
        		target="DesktopDeleteLabel"; //$NON-NLS-1$
        		
        	} else if (command.equals("DesktopListAllLabel")) { //$NON-NLS-1$
        		target="DesktopListAllLabel"; //$NON-NLS-1$


        	} else if (command.equals("DesktopMenu")) { //$NON_NLS-1$
        		target="GotoDesktopMenu";
        	}
        	
        } else {
        	errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.UNKNOWN_SCREEN_4, request.getRequestURI(), screen, command, this.getClass().getName()));
        }


    	if (target == null) {
    		if (command.equals("MainMenu")) { //$NON-NLS-1$
    			target="GotoMainMenu"; //$NON-NLS-1$
    		}
    	}

        if (target != null) {
        	forward = mapping.findForward(target);
        	request.setAttribute(Constants.FORM_DATA_ATTR, formData);
        } else {
        	errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.UNKNOWN_COMMAND_4, request.getRequestURI(), screen, command, this.getClass().getName()));
        }
        
        log.exiting(getClass().getName(), "execute", "target=" + target + "; forward=" + forward);//.getName()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        
        // Finish with
        return (forward);

    }
}
