/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.actions;

import java.util.ArrayList;
import java.util.Map;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.ibm.trackntrace.forms.FormData;
import com.ibm.trackntrace.usecases.Sampling;
import com.ibm.trackntrace.usecases.UseCaseException;
import com.ibm.trackntrace.usecases.data.PharmaUnitInfo;
import com.ibm.trackntrace.utils.BarcodeUtils;
import com.ibm.trackntrace.utils.Constants;
import com.ibm.trackntrace.utils.RoleDefs;
import com.ibm.trackntrace.utils.RoleValidation;

/**
 * This class implments the actions for the Sampling use case screens.
 * 
 */
public class SamplesAction extends BXActionBase {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";

	private static final String MAIN_MENU = "GotoMainMenu";

	private static final String SAMPLES_MENU = "SamplesMenu";

	private static final String GET_CASE_PAGE = "SamplesScanCase";

	private static final String GET_UNIT_PAGE = "SamplesScanUnit";

	private static final String TAG_DETAILS_MAP = "SamplingTagDetailsMap";

	static {
		log = Logger.getLogger(Constants.LOG_SAMPLES);
	}

	/**
	 * Handles the logic a unit/case tag. Retrives the product information from
	 * the backend
	 * 
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processsTag(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {

		String target = null;
		String thisUrn = null;
		String thisTag = null;

		log.entering(getClass().getName(), "processTag", formData); //$NON-NLS-1$

		// get current barcode scan
		thisTag = formData.getSerialNumber();

		if (thisTag != null) {
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);

		} else {
			log.warning("Passed in barcode is null!");
			// formData.setLastError("Null Barcode scanned in");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.NULL_BARCODE, this.getClass().getName()));
			target = REDISPLAY;
		}
		/*
		 * Set the case/unit tag as the current tag. The same code executes for
		 * either a unit or a case, not at the same time
		 */
		formData.setCaseTag(thisTag);
		formData.setUnitTag(thisTag);

		// if barcode parse fails, return.
		if (thisUrn == null) {
			log
					.finest("convertBarcodeToEPCpureID returned an null urn for barcode="
							+ thisTag);
			target = REDISPLAY;
			return target;
		}

		// add the barcode to session data and get the barcode product details
		// from backend.
		if (addBarCodeDetailsToSession(tagDetailsMap, thisTag, thisUrn,
				TAG_DETAILS_MAP, formData,
				new Sampling(formData.getStation(), formData.getBizLocation(),
						formData.getTimeZoneOffset()), false)) {
			log.finest("Size of " + TAG_DETAILS_MAP + " is "
					+ tagDetailsMap.size());
			// we don't know if this is a unit or a case, so set both to the
			// same value
			formData.setCaseCount(tagDetailsMap.size());
			formData.setUnitCount(tagDetailsMap.size());
			// render the product data for the barcode.
			buildDisplayData(tagDetailsMap, formData, errors);
		} else {
			log
					.fine("addBarCodeDetailsToSession failed! setting display to current screen");
			target = REDISPLAY;
		}
		log.exiting(getClass().getName(), "processTag", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * Handles confirmation of sampling.
	 * 
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processScanDone(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {
		String target = null;
		log
				.entering(
						getClass().getName(),
						"processScanDone", new Object[] { tagDetailsMap, formData, errors }); //$NON-NLS-1$

		// submit the tag list to the backend.
		try {
			Sampling sampling = new Sampling(formData.getStation(), formData
					.getBizLocation(), formData.getTimeZoneOffset());
			Object[] barcodes = tagDetailsMap.keySet().toArray();
			log.finest("Number of tags to be sampled = " + barcodes.length);
			for (int i = 0; i < barcodes.length; ++i) {
				String barcode = (String) barcodes[i];
				log.fine("barcode = " + barcode);
				PharmaUnitInfo unitInfo = tagDetailsMap.get(barcode);
				if (unitInfo == null) {
					log.severe("Retreived a null unitInfo object from "
							+ TAG_DETAILS_MAP + " for barcode = " + barcode);
					// formData.setLastError("Serious error occured. Report to
					// Administrator");
					errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
							Constants.SESSION_DATA_ERROR, this.getClass()
									.getName()));
					return target;
				}
				log.fine("unitInfo=" + unitInfo.toString());
				/*
				 * Since the usecase allows a case or a unit to be sampled We
				 * ned to figure out which one is actually being sampled. If
				 * they are sampling units, the units count will be greater then
				 * 0 if cases are being sampled, cases count will be greater the
				 * 0. The users can't mix and match cases and units during
				 * Sampling
				 */
				String sampledURN = null;
				if (formData.getCaseCount() > 0) {
					sampledURN = unitInfo.getUnitURN();
					log.fine("Sampling case " + sampledURN);
				} else {
					sampledURN = unitInfo.getUnitURN();
					log.fine("Sampling unit " + sampledURN);
				}
				if (sampling.confirmSampling(sampledURN)) {
					log.fine("Sampling confirmed with backend for URN"
							+ barcode);
					// clear the current tag from session
					this.removeBarCodeDetailsFromSession(tagDetailsMap,
							formData, barcode, TAG_DETAILS_MAP);
				} else {
					log
							.severe("Unable to confirm sampling with backend for URN = "
									+ barcode);
					// formData.setLastError("Unable to sample with backend.
					// Please report to administrator");
					errors.add(ActionErrors.GLOBAL_MESSAGE,
							new ActionError(Constants.CONFIRM_FAILED, this
									.getClass().getName()));
					target = REDISPLAY;
					break;
				}
			}
		} catch (UseCaseException e) {
			log.severe("Caught use case exception during Sampling. "
					+ e.getMessage());
			e.printStackTrace();
			target = REDISPLAY;
			// formData.setLastError("Serious error occured. Please report to
			// Administrator");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.BACKEND_ERROR, this.getClass().getName()));

		}
		log
				.exiting(getClass().getName(),
						"processScanDone", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * Handles the undo button press on the GUI by removing the last item
	 * scanned.
	 * 
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processUndo(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {

		String target = SAMPLES_MENU;
		String lastBarcode = null;
		log.entering(getClass().getName(), "processUndo", formData); //$NON-NLS-1$

		if (tagDetailsMap == null || formData == null) {
			log
					.severe("Input arguments are not as expected. Null argument(s) passed in");
			return target;
		}
		// get the last barcode from the session map
		if (tagDetailsMap.size() != 0) {
			// get the last entry in the map, this is a LinkedHashMap, so the
			// keys are ordered by insertion
			Object[] barcodes = tagDetailsMap.keySet().toArray();
			if (barcodes.length > 0) {
				lastBarcode = (String) barcodes[barcodes.length - 1];
				if (lastBarcode != null) {
					// remove the details from the session
					if (!removeBarCodeDetailsFromSession(tagDetailsMap,
							formData, lastBarcode, TAG_DETAILS_MAP)) {
						target = REDISPLAY;
						return target;
					} else {

						// get new barcode list
						Object[] newBarcodeList = tagDetailsMap.keySet()
								.toArray();
						if (formData.getCaseCount() > 0) {
							formData.setCaseCount(tagDetailsMap.size());
							if (newBarcodeList.length > 0) {
								formData
										.setCaseTag((String) newBarcodeList[newBarcodeList.length - 1]);
								target = GET_CASE_PAGE;
							}

						} else if (formData.getUnitCount() > 0) {
							formData.setUnitCount(tagDetailsMap.size());

							if (newBarcodeList.length > 0) {
								formData
										.setUnitTag((String) newBarcodeList[newBarcodeList.length - 1]);
								target = GET_UNIT_PAGE;
							}

						} else {
							target = SAMPLES_MENU;
						}
						// change display to reflect the undo.
						buildDisplayData(tagDetailsMap, formData, errors);
					}
				}
			}
		} else {
			log.warning("Nothing to undo!");
			// formData.setLastError("Nothing to undo!");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.NOTHING_TO_UNDO, this.getClass().getName()));
		}

		log.exiting(getClass().getName(), "processUndo", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected ActionForward execute(ActionMapping mapping, FormData formData,
			HttpServletRequest request, HttpServletResponse response,
			ActionErrors errors) throws Exception {
		ActionForward forward = new ActionForward(); // return value
		String target = null;
		String screen = formData.getScreen();
		String command = formData.getCommand();

		log.entering(getClass().getName(), "execute", formData); //$NON-NLS-1$

		// Strip barcode of prefix
		formData.setSerialNumber(BarcodeUtils.stripBarcodePrefix(formData
				.getSerialNumber()));

		Map<String, PharmaUnitInfo> tagDetailsMap = createTagDetailsMapInSession(
				request, TAG_DETAILS_MAP);

		if (RoleValidation.validate(request, RoleDefs.SAMPLER) == false) {
			target = "GotoMainMenu";

		} else if (command.equals("GotCase")) { //$NON-NLS-1$
			target = this.processsTag(tagDetailsMap, formData, errors);
			if (target == null) {
				target = GET_CASE_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_CASE_PAGE;
			}

		} else if (command.equals("GotUnit")) { //$NON-NLS-1$
			target = this.processsTag(tagDetailsMap, formData, errors);
			if (target == null) {
				target = GET_UNIT_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_UNIT_PAGE;
			}

		} else if (command.equals("Done")) { //$NON-NLS-1$
			target = processScanDone(tagDetailsMap, formData, errors);
			if (target == null) {
				target = SAMPLES_MENU;
			} else {
				// formData.setLastError("processScanDone failed.");
				errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
						Constants.CONFIRM_FAILED, this.getClass().getName()));
				if (formData.getCaseCount() > 0) {
					target = GET_CASE_PAGE;
				} else if (formData.getUnitCount() > 0) {
					target = GET_UNIT_PAGE;
				} else {
					target = SAMPLES_MENU;
				}
			}

		} else if (command.equals("Undo")) { //$NON-NLS-1$
			target = processUndo(tagDetailsMap, formData, errors);
			if (target == null) {
				formData.setLastError("processUndo failed.");
				errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
						Constants.UNDO_FAILED, this.getClass().getName()));
				target = SAMPLES_MENU;
			} else if (target.equals(REDISPLAY)) {
				target = SAMPLES_MENU;
			}

		} else if (command.equals("Cancel")) { //$NON-NLS-1$
			// clear the tag details map.
			log.fine("Clearing the " + TAG_DETAILS_MAP + " . Current size is "
					+ tagDetailsMap.size());
			tagDetailsMap.clear();
			target = SAMPLES_MENU;

		} else if (command.equals("MainMenu")) { //$NON-NLS-1$
			target = MAIN_MENU;
		}

		if (target != null) {
			forward = mapping.findForward(target);
			request.setAttribute(Constants.FORM_DATA_ATTR, formData);
		} else {
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.UNKNOWN_COMMAND_4, request.getRequestURI(),
					screen, command, this.getClass().getName()));
		}

		log.exiting(getClass().getName(),
				"execute", "target=" + target + "; forward=" + forward); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// Finish with
		return (forward);
	}

	/**
	 * Build the display string to render the barcode product information.
	 * 
	 * @param map
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String buildDisplayData(Map<String, PharmaUnitInfo> map,
			FormData formData, ActionErrors errors) {

		log.entering(getClass().getName(), "buildDisplayData", new Object[] {
				map, formData, errors });

		String target = null;

		ArrayList<String> ndc = new ArrayList<String>();
		ArrayList<String> des = new ArrayList<String>();

		Object[] unitInfoArray = (Object[]) map.values().toArray();

		for (int i = 0; i < unitInfoArray.length; ++i) {
			PharmaUnitInfo unit = (PharmaUnitInfo) unitInfoArray[i];
			ndc.add(unit.getNDC());
			des.add(unit.getProductName());
		}

		String display = "<p><table class=\"scanData\">"
				+ "<tr><th width=70%>NDC</th>" + "</tr>"
				+ "<tr><th colspan=3>Item Description</th></tr>";
		final int j = 0x0001;

		for (int i = 0; i < unitInfoArray.length; i++) {
			int k = i & j;
			if (k == 0) {
				display = display + "<tr class=\"even\"><td>";
			} else {
				display = display + "<tr><td>";
			}
			display = display + ndc.get(i) + "</td></tr>";

			if (k == 0) {
				display = display + "<tr class=\"even\">";
			} else {
				display = display + "<tr>";
			}
			display = display + "<td colspan=3>" + des.get(i) + "</td></tr>";
		}
		display = display + "</td></tr></table>";
		formData.setDisplayData(display);

		log.exiting(getClass().getName(), "buildDisplayData", target);
		return target;
	}
}
