/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.actions;

import java.util.ArrayList;
import java.util.Map;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.ibm.trackntrace.forms.FormData;
import com.ibm.trackntrace.usecases.SplitPallet;
import com.ibm.trackntrace.usecases.UseCaseException;
import com.ibm.trackntrace.usecases.data.PharmaUnitInfo;
import com.ibm.trackntrace.utils.BarcodeUtils;
import com.ibm.trackntrace.utils.Constants;
import com.ibm.trackntrace.utils.RoleDefs;
import com.ibm.trackntrace.utils.RoleValidation;

public class SplitPalletAction extends BXActionBase {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";

	private static final String MAIN_MENU = "GotoMainMenu";

	private static final String GET_PALLET_PAGE = "GotoMainMenu";

	private static final String SCAN_CASE_PAGE = "SplitPalletScanCase";

	private static final String GET_SSC_PAGE = "SplitPalletGetSSCCtag";

	private static final String TAG_DETAILS_MAP = "SplitPalletTagDetailsMap";

	static {
		log = Logger.getLogger(Constants.LOG_SPLIT);
	}

	protected String processPalletTag(FormData formData, ActionErrors errors) {

		String target = null;
		String thisUrn = null;
		String thisTag = null;

		log.entering(getClass().getName(),
				"processPalletTag", new Object[] { formData, errors }); //$NON-NLS-1$

		// get current barcode scan
		thisTag = formData.getSerialNumber();

		if (thisTag != null) {
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);
		} else {
			log.warning("Passed in barcode is null!");
			formData.setLastError("Null Barcode scanned in");
			target = REDISPLAY;
			return target;
		}

		// if barcode parse fails, return.
		if (thisUrn == null) {
			log
					.finest("convertBarcodeToEPCpureID returned an null urn for barcode="
							+ thisTag);
			target = REDISPLAY;
			return target;
		}

		// save the pallet barcode and urn in the form,
		formData.setPalletTag(thisTag);
		formData.setPalletUrn(thisUrn);
		target = SCAN_CASE_PAGE;

		log.exiting(getClass().getName(), "processPalletTag", target);
		return target;
	}

	protected String processCaseTag(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {
		String target = null;
		String thisUrn = null;
		String thisTag = null;

		log
				.entering(
						getClass().getName(),
						"processCaseTag", new Object[] { tagDetailsMap, formData, errors }); //$NON-NLS-1$

		// get current barcode scan
		thisTag = formData.getSerialNumber();

		if (thisTag != null) {
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);
		} else {
			log.warning("Passed in barcode is null!");
			formData.setLastError("Null Barcode scanned in");
			target = REDISPLAY;
			return target;
		}

		// if barcode parse fails, return.
		if (thisUrn == null) {
			log
					.finest("convertBarcodeToEPCpureID returned an null urn for barcode="
							+ thisTag);
			target = REDISPLAY;
			return target;
		}

		// add the barcode to session data and get the barcode product details
		// from backend.
		if (addBarCodeDetailsToSession(tagDetailsMap, thisTag, thisUrn,
				TAG_DETAILS_MAP, formData, new SplitPallet(formData
						.getStation(), formData.getBizLocation(), formData
						.getTimeZoneOffset()), false)) {

			formData.setCaseCount(tagDetailsMap.size());
			formData.setCaseTag(thisTag);

			// render the product data for the barcode.
			buildDisplayData(tagDetailsMap, formData, errors);
		} else {
			log
					.fine("addBarCodeDetailsToSession failed! setting display to current screen");
			formData
					.setLastError("Unable to retrieve barcode details from backend!");
			target = REDISPLAY;
		}
		log.exiting(getClass().getName(), "processTag", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * Processes the Done button being pressed on any screen. All tags scanned
	 * in that screen along with the parent tag are submitted to the backend for
	 * processing
	 * 
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processScanDone(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {
		String target = null;
		log.entering(getClass().getName(), "processConfirm", formData); //$NON-NLS-1$

		// submit the tag list to the backend.
		try {

			Object[] barcodes = tagDetailsMap.keySet().toArray();
			ArrayList<String> caseURNs = new ArrayList<String>();
			String palletURN = formData.getPalletUrn();
			for (int i = 0; i < barcodes.length; ++i) {
				String barcode = (String) barcodes[i];
				PharmaUnitInfo unitInfo = tagDetailsMap.get(barcode);
				log.finest("Retrieved unitInfo=" + unitInfo + " for barcode "
						+ barcode);
				caseURNs.add(unitInfo.getParentURN());
			}

			// if there are childrent to be recommissioned, send them to the
			// backend
			// along with parent to be aggregated with.
			if (caseURNs.size() > 0) {
				SplitPallet splitPallet = new SplitPallet(
						formData.getStation(), formData.getBizLocation(),
						formData.getTimeZoneOffset());
				if (splitPallet.confirmSplitPallet(caseURNs, formData
						.getPalletUrn())) {
					log.fine("Pallet split succeeded for pallet URN = "
							+ palletURN + " and childURNs= "
							+ caseURNs.toString());
					tagDetailsMap.clear();
				} else {
					log.severe("Split pallet failed for for parentURN = "
							+ palletURN + " and childURNs= "
							+ caseURNs.toString());
					formData
							.setLastError("Split Pallet Failed! Report to Administrator");
				}
			}

		} catch (UseCaseException e) {
			log.severe("Caught use case exception during Sampling. "
					+ e.getMessage());
			e.printStackTrace();
			target = REDISPLAY;
			formData
					.setLastError("Serious error occured. Please report to Administrator");

		}

		log.exiting(getClass().getName(), "processConfirm", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected String clearForm(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {
		String target = null;
		log.entering(getClass().getName(), "clearForm", formData); //$NON-NLS-1$

		formData.setTagList(null);
		formData.setCaseTag("");
		formData.setCaseCount(0);
		formData.setSerialNumber("");
		formData.setPalletTag("");

		if (tagDetailsMap != null) {
			tagDetailsMap.clear();
		}

		log.exiting(getClass().getName(), "clearForm", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * Handles the undo button press on the GUI by removing the last item
	 * scanned.
	 * 
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processUndo(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {

		String target = SCAN_CASE_PAGE;
		String lastBarcode = null;
		log.entering(getClass().getName(), "processUndo", formData); //$NON-NLS-1$

		if (tagDetailsMap == null || formData == null) {
			log
					.severe("Input arguments are not as expected. Null argument(s) passed in");
			return target;
		}
		// get the last barcode from the session map
		if (tagDetailsMap.size() != 0) {
			// get the last entry in the map, this is a LinkedHashMap, so the
			// keys are ordered by insertion
			Object[] barcodes = tagDetailsMap.keySet().toArray();
			if (barcodes.length > 0) {
				lastBarcode = (String) barcodes[barcodes.length - 1];
				if (lastBarcode != null) {
					// remove the details from the session
					if (!removeBarCodeDetailsFromSession(tagDetailsMap,
							formData, lastBarcode, TAG_DETAILS_MAP)) {
						target = REDISPLAY;
						return target;
					} else {

						// get new barcode list
						Object[] newBarcodeList = tagDetailsMap.keySet()
								.toArray();
						formData.setCaseCount(tagDetailsMap.size());
						if (newBarcodeList.length > 0) {
							formData
									.setCaseTag((String) newBarcodeList[newBarcodeList.length - 1]);
						} else {
							formData.setCaseTag("");
						}
						// change display to reflect the undo.
						buildDisplayData(tagDetailsMap, formData, errors);
					}
				}
			}
		} else {
			log.warning("Nothing to undo!");
			formData.setLastError("Nothing to undo!");
		}

		log.exiting(getClass().getName(), "processUndo", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected ActionForward execute(ActionMapping mapping, FormData formData,
			HttpServletRequest request, HttpServletResponse response,
			ActionErrors errors) throws Exception {
		ActionForward forward = new ActionForward(); // return value
		String target = null;
		String screen = formData.getScreen();
		String command = formData.getCommand();

		log.entering(getClass().getName(), "execute", formData); //$NON-NLS-1$

		// Strip barcode of prefix
		formData.setSerialNumber(BarcodeUtils.stripBarcodePrefix(formData
				.getSerialNumber()));

		// get the barcode details from the session
		Map<String, PharmaUnitInfo> tagDetailsMap = createTagDetailsMapInSession(
				request, TAG_DETAILS_MAP);

		System.out.println("Command = " + formData.getCommand());

		if (RoleValidation.validate(request, RoleDefs.SPLITTER) == false) {
			target = "GotoMainMenu";

		} else if (command.equals("Skip")) { //$NON-NLS-1$
			target = GET_PALLET_PAGE;
			clearForm(tagDetailsMap, formData, errors);
			// formData.setPalletTag("-1");

		} else if (command.equals("GotPallet")) { //$NON-NLS-1$
			target = processPalletTag(formData, errors);
			if (target == null) {
				target = SCAN_CASE_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_PALLET_PAGE;
			}

		} else if (command.equals("GotCase")) { //$NON-NLS-1$
			target = processCaseTag(tagDetailsMap, formData, errors);
			if (target == null) {
				target = SCAN_CASE_PAGE;
			} else {
				formData.setLastError("processCaseTag failed.");
				target = SCAN_CASE_PAGE;
			}

		} else if (command.equals("Done")) { //$NON-NLS-1$
			target = processScanDone(tagDetailsMap, formData, errors);
			if (target == null) {
				target = GET_SSC_PAGE;
			}

		} else if (command.equals("Cancel")) { //$NON-NLS-1$
			clearForm(tagDetailsMap, formData, errors);
			target = MAIN_MENU;

		} else if (command.equals("Undo")) { //$NON-NLS-1$

			target = processUndo(tagDetailsMap, formData, errors);
			if (target == null) {
				formData.setLastError("processUndo failed.");
				target = SCAN_CASE_PAGE;
			}

		} else if (command.equals("MainMenu")) { //$NON-NLS-1$
			target = MAIN_MENU;
		}

		if (target != null) {
			forward = mapping.findForward(target);
			request.setAttribute(Constants.FORM_DATA_ATTR, formData);
		} else {
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.UNKNOWN_COMMAND_4, request.getRequestURI(),
					screen, command, this.getClass().getName()));
		}

		log.exiting(getClass().getName(),
				"execute", "target=" + target + "; forward=" + forward); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// Finish with
		return (forward);
	}

	/**
	 * Build the display string to render the barcode product information.
	 * 
	 * @param map
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String buildDisplayData(Map<String, PharmaUnitInfo> map,
			FormData formData, ActionErrors errors) {

		log.entering(getClass().getName(), "buildDisplayData", new Object[] {
				map, formData, errors });

		String target = null;

		ArrayList<String> ndc = new ArrayList<String>();
		ArrayList<String> des = new ArrayList<String>();

		Object[] unitInfoArray = (Object[]) map.values().toArray();

		if (unitInfoArray.length != 0) {

			for (int i = 0; i < unitInfoArray.length; ++i) {
				PharmaUnitInfo unit = (PharmaUnitInfo) unitInfoArray[i];
				ndc.add(unit.getNDC());
				des.add(unit.getProductName());
			}

			String display = "<p><table class=\"scanData\">"
					+ "<tr><th width=70%>NDC</th>" + "</tr>"
					+ "<tr><th colspan=3>Item Description</th></tr>";
			final int j = 0x0001;

			for (int i = 0; i < unitInfoArray.length; i++) {
				int k = i & j;
				if (k == 0) {
					display = display + "<tr class=\"even\"><td>";
				} else {
					display = display + "<tr><td>";
				}
				display = display + ndc.get(i) + "</td></tr>";

				if (k == 0) {
					display = display + "<tr class=\"even\">";
				} else {
					display = display + "<tr>";
				}
				display = display + "<td colspan=3>" + des.get(i)
						+ "</td></tr>";
			}
			display = display + "</td></tr></table>";
			formData.setDisplayData(display);
		} else {
			// nothing to display
			log
					.warning("TagDetailsMap is empty! Setting form unit information display data to null");
			formData.setDisplayData("");
		}

		log.exiting(getClass().getName(), "buildDisplayData", target);
		return target;
	}
}
