/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.usecases;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import com.ibm.premises.reusable.constants.Constants;
import com.ibm.premises.reusable.constants.Constants.ObjectEventConstants;
import com.ibm.trackntrace.utils.Constants.BusinessSteps;
import com.ibm.trackntrace.utils.Constants.Disposition;

/**
 * @author Sudhir Pucha This classes defines methods that implement the
 *         Receiving use case.
 * 
 */
public class Receiving extends AbstractUseCase {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";

	/**
	 * constructor
	 * 
	 * @param location
	 */
	public Receiving(String userAgentID, String bizLoc, String tzOffset) {
		this.location = userAgentID;
		this.businessLocation = bizLoc;
		this.timeZoneOffset = tzOffset;
	}

	/**
	 * class name, used for logging
	 */
	private String clsName = Receiving.class.getName();

	/**
	 * the logger for this class
	 */
	private static Logger theLogger = Logger.getLogger(Receiving.class
			.getName());

	/**
	 * Accept the pallet. This methods interfacts with the Premises Server RUCs
	 * to indicate the pallet has been received resulting in an Observe message
	 * to be sent to the backend RFIDIC/EPCIS.
	 * 
	 * @return true/false to indicate sucess/failure of confirm
	 * @throws UseCaseException
	 */
	public boolean confirmPalletReceive(String urn) throws UseCaseException {

		methodName = "confirmPalletReceive";
		theLogger.entering(clsName, methodName, urn);

		// call Inference to get leaf children of the tag ( units)

		ArrayList<String> allTags = new ArrayList<String>();
		allTags = this.getChildren(urn, false, true);

		// get cases. as leaf children don't return cases

		ArrayList<String> caseTags = this.getChildren(urn, false, false);

		if (caseTags != null) {
			// add cases to alltags
			allTags.addAll(caseTags);
		}
		if (urn != null) {
			// add the pallet to the list of tags.
			allTags.add(urn);
		}

		Map<String, String> metaData = new HashMap<String, String>();
		metaData.put(ObjectEventConstants.BIZSTEP_KEY_OUT,
				BusinessSteps.RECEIVING);
		metaData.put(ObjectEventConstants.DISPOSITION_KEY_OUT,
				Disposition.IN_PROGRESS);
		metaData.put(ObjectEventConstants.READPOINT_KEY_OUT, this.location);

		// get the orderID from the backend if available
		String bizTransaction = this.getBizTrasactionByEPC(urn);
		if (bizTransaction != null) {
			metaData
					.put(
							ObjectEventConstants.BIZTRANSACTIONTYPE_KEY_OUT,
							com.ibm.trackntrace.utils.Constants.RFIDIC_BIZ_TRANSACTION_TYPE);
			metaData.put(ObjectEventConstants.BIZTRANSACTION_KEY_OUT,
					bizTransaction);
		} else {
			theLogger
					.warning("BizTransaction/orderID unavailble or no previous transfer/shipping event for urn="
							+ urn);
		}

		if (this.businessLocation != null) {
			metaData.put(Constants.ObjectEventConstants.BIZLOCATION_KEY_OUT,
					this.businessLocation);
		} else {
			theLogger
					.warning("Business location information missing from WAS resource environment");
		}
		metaData.put(
				Constants.ObjectEventConstants.EVENT_TIME_ZONE_OFFSET_KEY_OUT,
				this.timeZoneOffset);

		if (allTags != null && allTags.size() > 0) {
			// Observe
			this.observe(allTags, metaData);
		}

		theLogger.exiting(clsName, methodName, urn);
		return true;
	}
}
