/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.usecases;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import com.ibm.premises.reusable.constants.Constants;
import com.ibm.trackntrace.utils.Constants.BusinessSteps;
import com.ibm.trackntrace.utils.Constants.Disposition;

public class Recommission extends AbstractUseCase {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";
	
	private String clsName = Recommission.class.getName();

	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger.getLogger(Recommission.class
			.getName());

	/**
	 * Create the Recommissioning object
	 * @param locationID
	 */
	public Recommission(String locationID, String bizLoc, String tzOffset) {
		this.location = locationID;
		this.businessLocation = bizLoc;
		this.timeZoneOffset = tzOffset;
	}

	/**
	 * Confirm recommisoining by Observing the case with units as active thereby
	 * removing non_sellableness. Also, aggregate the childURNs to the parent.
	 * 
	 * @param caseURN
	 * @return
	 * @throws UseCaseException
	 */
	public boolean confirmRecommission(	String parentURN, ArrayList<String> childURNs) throws UseCaseException {

		boolean result = false;
		theLogger.entering(clsName, methodName, new Object[] { parentURN, childURNs });

		/*
		 * Observe the parent and ChildURNs. Aggreate the childURNs to the
		 * Parent.
		 */

		// get immediate children of the case
		ArrayList<String> tags = new ArrayList<String>();
		Map<String, String> metaData = new HashMap<String, String>();
		metaData.put(Constants.ObjectEventConstants.BIZSTEP_KEY_OUT,
				BusinessSteps.INSPECTING);
		metaData.put(Constants.ObjectEventConstants.DISPOSITION_KEY_OUT,
				Disposition.IN_PROGRESS);
		metaData.put(Constants.ObjectEventConstants.READPOINT_KEY_OUT,
				this.location);
	
		if (this.businessLocation != null) {
			metaData.put(Constants.ObjectEventConstants.BIZLOCATION_KEY_OUT,
					this.businessLocation);
		} else {
			theLogger
					.warning("Business location information missing from WAS resource environment");
		}
		metaData.put(
				Constants.ObjectEventConstants.EVENT_TIME_ZONE_OFFSET_KEY_OUT,
				this.timeZoneOffset);
		
		if (parentURN != null && childURNs != null) {
			tags = childURNs;
			tags.add(parentURN);
		}

		if (tags != null && tags.size() > 0) {
			ArrayList<String> tags2Observe = new ArrayList<String>();
			tags2Observe = tags;
			// observe as active
			// also retrive any children of the child URNs. or examples when recommissiong
			// a case, the units need to be observed too
			for(int i=0; i < childURNs.size() ; ++i){
				String child = childURNs.get(i);
				if( child != null){
					ArrayList<String> grandChildren = getChildren(child, false, false);
					if( grandChildren != null && grandChildren.size() > 0){
						tags2Observe.addAll(grandChildren);
					}
				}
			}
			observe(tags2Observe, metaData);
			
			//same metadata as Observe, just add the PARENTTAG info.
			//aggregate the child to parent
			metaData.put("PARENTTAG", parentURN);
			aggregate(tags, metaData);

			result = true;
		}
		theLogger.exiting(clsName, methodName, result);
		return result;

	}
}
