/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.usecases;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import com.ibm.premises.reusable.constants.Constants;
import com.ibm.premises.reusable.constants.Constants.ObjectEventConstants;
import com.ibm.trackntrace.utils.Constants.BusinessSteps;
import com.ibm.trackntrace.utils.Constants.Disposition;

/**
 * @author Sudhir Pucha
 *
 */
public class Returns extends AbstractUseCase {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";
	
	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger.getLogger(Returns.class
			.getName());
	
	/**
	 *  class name = used for logging
	 */
	private String clsName = Returns.class.getName();
	
	/**
	 * This element describes the valid dispostions for this usecase.
	 *
	 */
	public enum ReturnsDisposiiton{
		Return, Destory;
	}
	
	/**
	 * @param locationID
	 */
	public Returns( String locationID, String bizLoc, String tzOffset){
		this.location = locationID;
		this.businessLocation = bizLoc;
		this.timeZoneOffset = tzOffset;
	}
	
	/**
	 * confirm disposition with the backend.
	 * @param tagList
	 * @param disp
	 * @return
	 * @throws UseCaseException
	 */
	public boolean confirmReturnsDisposition( ArrayList<String> tagList , ReturnsDisposiiton disp)throws UseCaseException {
		
		methodName = "confirmReturnsDisposition";
		boolean result = false;
		
		theLogger.entering(clsName, methodName, new Object[]{ tagList.toString(), disp});
		
		//for each tag in the list
		//get all the children 
		//decommission tag + children
		
		for ( int i=0; i< tagList.size() ; ++i){
			
			String urn = null;
			urn = tagList.get(i);
			theLogger.finest("urn = "+ urn);
			
			ArrayList<String> childTags = null;
			
			if( urn != null){
				childTags = getChildren(urn, false, false);
				ArrayList<String> tagsToObserve = new ArrayList<String>();
				if( childTags != null){
					// add the child tags + the current urn to list of tags to be Observed.
					tagsToObserve.addAll(childTags);
					tagsToObserve.add(urn);
					theLogger.finest("CaseTag: Tags to Observe = " +  tagsToObserve.toString());
				}
				else{
					// no child tags, this must be a unit tag
					// observe the unit based on disposition
					tagsToObserve.add(urn);
					theLogger.finest("UnitTag: Tags to Observe = " +  tagsToObserve.toString());
				}
				Map<String, String> metaData = new HashMap<String, String>();
				metaData.put(
						Constants.ObjectEventConstants.BIZSTEP_KEY_OUT,
						BusinessSteps.RECEIVING);
				
				metaData.put(
						Constants.ObjectEventConstants.READPOINT_KEY_OUT,
						this.location);

				
				if(disp == ReturnsDisposiiton.Destory ) {
					metaData.put(
							Constants.ObjectEventConstants.DISPOSITION_KEY_OUT,
							Disposition.DESTROYING);
				}
				else {
					metaData.put(
							Constants.ObjectEventConstants.DISPOSITION_KEY_OUT,
							Disposition.RETURNED);
				}
				
				
				if (this.businessLocation != null) {
					metaData.put(Constants.ObjectEventConstants.BIZLOCATION_KEY_OUT,
							this.businessLocation);
				} else {
					theLogger
							.warning("Business location information missing from WAS resource environment");
				}
				
				metaData.put(
						Constants.ObjectEventConstants.EVENT_TIME_ZONE_OFFSET_KEY_OUT,
						this.timeZoneOffset);
				
				//get the orderID from the backend if available
				String bizTransaction = this.getBizTrasactionByEPC(urn);
				if ( bizTransaction != null){
					metaData.put(ObjectEventConstants.BIZTRANSACTIONTYPE_KEY_OUT,com.ibm.trackntrace.utils.Constants.RFIDIC_BIZ_TRANSACTION_TYPE);
					metaData.put(ObjectEventConstants.BIZTRANSACTION_KEY_OUT, bizTransaction);
				}
				else{
					theLogger.warning("BizTransaction/orderID unavailble or no previous transfer/shipping event for urn=" + urn);
				}
				
				//	observe
				observe(tagsToObserve, metaData);

				result = true;
			}
		}	
		return result;
	}

}
