/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.usecases.data;

import java.util.ArrayList;

/**
 * @author Sudhir Pucha
 * This class describes a container of type case.
 * A homogenous case will have information about a single unit.
 * A non-homogenous case will describe all units in it.
 *
 */
public class CaseInfo {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";
	
	public CaseInfo(String urn){
		this.caseURN = urn;
		units = new ArrayList<PharmaUnitInfo>();
		
	}
	public CaseInfo(){
		units = new ArrayList<PharmaUnitInfo>();
	}
	/**
	 *  number of units in the cases
	 */
	private int unitsPerCase = 0;
	
	/**
	 * EPC URN of the case. Pure ID form.
	 */
	private String caseURN = null;
	
	/**
	 * Parent tag.
	 */
	private String parentURN = null;
	
	/**
	 * This flag indicates if a case is homogenous.
	 */
	private boolean homogenous = true;
	
	/**
	 *  Unit Information. 
	 */
	private ArrayList<PharmaUnitInfo> units;

	/**
	 * @return
	 */
	public String getCaseURN() {
		return caseURN;
	}

	/**
	 * @param caseURN
	 */
	public void setCaseURN(String caseURN) {
		this.caseURN = caseURN;
	}

	/**
	 * @return
	 */
	public boolean isHomogenous() {
		return homogenous;
	}
	/**
	 * @return
	 */
	public int getUnitsPerCase() {
		return unitsPerCase;
	}

	/**
	 * @param unitsPerCase
	 */
	public void setUnitsPerCase(int unitsPerCase) {
		this.unitsPerCase = unitsPerCase;
	}

	/**
	 * @return
	 */
	public ArrayList<PharmaUnitInfo> getUnits() {
		return units;
	}

	/**
	 * @param units
	 */
	public void setUnits(ArrayList<PharmaUnitInfo> units) {
		this.units = units;
	}

	/**
	 * @param homogenous
	 */
	public void setHomogenous(boolean homogenous) {
		this.homogenous = homogenous;
	}

	public String getParentURN() {
		return parentURN;
	}

	public void setParentURN(String parentURN) {
		this.parentURN = parentURN;
	}

	public void addUnitToCase(PharmaUnitInfo unitInfo){
		
		if( units != null)
			units.add(unitInfo);
		else {
			units = new ArrayList<PharmaUnitInfo>();
			units.add(unitInfo);
		}
		
		
	}



}
