/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.usecases.data;

import java.util.ArrayList;


/**
 * @author Sudhir Pucha
 * This class describes a pallet.
 * If a pallet is homogenous (indicated by isHomogenous()), only one
 * CaseInfo object is returned, else information about all cases on pallet is
 * returned.
 *
 */
public class PalletInfo {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";
	
	public PalletInfo(String urn){
		this.palletURN = urn;
		cases = new ArrayList<CaseInfo>();
	}
	
	/**
	 * number of cases in the pallet
	 */
	private int casesPerPallet = 0;
	
	/**
	 * number of units in the pallet
	 */
	private int unitsPerPallet = 0;
	
	/**
	 * pallet URN - pure id format
	 */
	private String palletURN = null;
	
	/**
	 * Is this a homogenous pallet. All cases and units
	 * are of the same type of drug.
	 */
	private boolean homogenous = true;
	
	/**
	 * case information of all cases on the pallet.
	 * array of CaseInfo objects
	 */
	private ArrayList<CaseInfo> cases = null;

	/**
	 * order information. purchase order/ transfer order.
	 */
	private OrderInfo order = null;

	/**
	 * @return
	 */
	public boolean isHomogenous() {
		return homogenous;
	}

	/**
	 * @param homogenous
	 */
	public void setHomogenous(boolean homogenous) {
		this.homogenous = homogenous;
	}

	
	/**
	 * @return pallet Tag in pure id form
	 */
	public String getPalletURN() {
		return palletURN;
	}

	/**
	 * @param palletURN
	 */
	public void setPalletURN(String palletURN) {
		this.palletURN = palletURN;
	}

	/**
	 * @return the list of cases.
	 */
	public ArrayList<CaseInfo> getCases() {
		return cases;
	}

	/**
	 * @param cases 
	 */
	public void setCases(ArrayList<CaseInfo> cases) {
		this.cases = cases;
	}

	/**
	 * @return
	 */
	public int getCasesPerPallet() {
		return casesPerPallet;
	}

	/**
	 * @param casesPerPallet
	 */
	public void setCasesPerPallet(int casesPerPallet) {
		this.casesPerPallet = casesPerPallet;
	}

	/**
	 * @return
	 */
	public OrderInfo getOrder() {
		return order;
	}

	/**
	 * @param order
	 */
	public void setOrder(OrderInfo order) {
		this.order = order;
	}

	/**
	 * @return
	 */
	public int getUnitsPerPallet() {
		return unitsPerPallet;
	}

	/**
	 * @param unitsPerPallet
	 */
	public void setUnitsPerPallet(int unitsPerPallet) {
		this.unitsPerPallet = unitsPerPallet;
	}
	
	public void addCaseToPallet(CaseInfo caseInfo){
		
		if( cases != null)
			cases.add(caseInfo);
		else {
			cases = new ArrayList<CaseInfo>();
			cases.add(caseInfo);
		}
	}

}
