/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.usecases.util;

import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;


import com.ibm.trackntrace.resources.lib.Config;
import com.ibm.trackntrace.usecases.AbstractUseCase;
import com.ibm.trackntrace.usecases.UseCaseException;

/**
 * @author Sudhir Pucha
 * This class looks up the custom properties added
 * to the Resource Environment in the WAS console.
 *
 */
public class PropertiesHelper {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";

	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger.getLogger(PropertiesHelper.class
			.getName());

	/**
	 * class name
	 */
	private String clsName = AbstractUseCase.class.getName();
	
	/**
	 * properties
	 */
	private Config properties = null;

	/**
	 * Constructor: lookup the properties file in WAS
	 * @throws UseCaseException
	 */
	public PropertiesHelper() throws UseCaseException {

		try {
			Context ctx = new InitialContext();
			this.properties = (Config) ctx
					.lookup("java:comp/env/PropertiesRef");
		} catch (NamingException e) {
			e.printStackTrace();
		}
	}

	
	/**
	 * Get the RFIDIC Query endpoint
	 * @return
	 */
	public String getRFIDICQueryEndpoint() {

		theLogger.entering(clsName, "getRFIDICQueryEndpoint");
		String endpoint = null;
		
		endpoint = (String) properties
				.getAttribute("RFIDIC_Query_Service_Endpoint");
		theLogger
				.fine("endpoint retreived from Resource Environment provider is ="
						+ endpoint);
		theLogger.exiting(clsName, "getRFIDICQueryEndpoint");
		
		return endpoint;

	}

}
