/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */
package com.ibm.trackntrace.usecases.util;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;


import com.ibm.sensorevent.model.ISensorEvent;

import com.ibm.sensorevent.model.converter.XMLConverter;
import com.ibm.sensorevent.model.generic.SensorEventException;
import com.ibm.sensorevent.util.SIBusPublisher;
import com.ibm.trackntrace.usecases.AbstractUseCase;

/**
 * @author Sudhir Pucha
 * Helper class to put and get messages off the SIBus.
 *
 */
public class SIBusHelper {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";

	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger.getLogger(AbstractUseCase.class
			.getName());

	/**
	 * variable that hold the class name of the object
	 */
	private static String clsName = AbstractUseCase.class.getName();

	/**
	 * variable to hold the method name
	 */
	protected static String methodName = null;

	/**
	 * @param topicName
	 * @param timeout
	 * @return
	 */
	public static ISensorEvent syncReceive(String connectionFactoryName, String topicName, long timeout) {

		methodName = "syncReceiveFromBus";
		theLogger.entering(clsName, methodName);
		
		ISensorEvent sensorEvent = null;

		Context jndiContext = null;
		TopicConnectionFactory topicConnectionFactory = null;
		Topic topic = null;
		TopicConnection topicConnection = null;
		TopicSession topicSession = null;
		TopicSubscriber topicSubscriber = null;

		try {
			jndiContext = new InitialContext();
			topicConnectionFactory = (TopicConnectionFactory) jndiContext
					.lookup(connectionFactoryName);
			topic = (Topic) jndiContext.lookup(topicName);
			topicConnection = topicConnectionFactory.createTopicConnection();
			topicSession = topicConnection.createTopicSession(false,
					Session.AUTO_ACKNOWLEDGE);
			topicSubscriber = topicSession.createSubscriber(topic);

			Message msg = topicSubscriber.receive(timeout);
			sensorEvent = IBMSensorEventHelper.convertToSensorEvent(msg);

		} catch (NamingException e) {
			theLogger.severe(e.getMessage());
		} catch (JMSException e) {
			theLogger.severe(e.getMessage());
		}
		
		theLogger.exiting(clsName, methodName);
		return sensorEvent;

	}

	/**
	 * @param eventType
	 * @param topic
	 * @param event
	 */
	public static void publishToBus(String eventType, String topic, ISensorEvent sensorEvent) {
		methodName = "publishToBus";
		theLogger.entering(clsName, methodName);

		// Print event XML to trace is set to finest
		if (Level.FINEST == theLogger.getLevel()) {
			XMLConverter converter = (XMLConverter) XMLConverter.getInstance();
			try {
				String xml = converter.toXMLString(sensorEvent);
				theLogger.finest("Publishing event to BUS --> " + xml);
			} catch (SensorEventException e) {
				e.printStackTrace();
			}
		}
		// Publish an ISensorEvent to the bus
		SIBusPublisher publisher = new SIBusPublisher();
		publisher.publish(eventType, topic, sensorEvent);
		theLogger.exiting(clsName, methodName);
	}
	

}
